/*******************************************************************************
* File Name: communication_api.c  
* Version 1.0
*
* Description:
* This file is created by the author . This contains definitions of APIs 
* used in structure 'CyBtldr_CommunicationsData' defined in cybtldr_api.h ,
* using SPI commuincations component 
********************************************************************************/

#include "communication_api.h"

/*******************************************************************************
* Function Name: OpenConnection
********************************************************************************
*
* Summary:
*  Initializes the communications component : In this case SPI
*
* Parameters:  
*  void
*
* Return: 
*  Returns a flag to indicate whether the operation was successful or not
*
*
*******************************************************************************/
int OpenConnection(void)
{
	SPIM_Start();
	return(CYRET_SUCCESS);
}


/*******************************************************************************
* Function Name: CloseConnection
********************************************************************************
*
* Summary:
*  Clears the status and stops the communications component (SPI).
*
* Parameters:  
*  void
*
* Return: 
*  Returns a flag to indicate whether the operation was successful or not
*
*
*******************************************************************************/
int CloseConnection(void)
{
	SPIM_Stop();
	return(CYRET_SUCCESS);
}

/*******************************************************************************
* Function Name: WriteData
********************************************************************************
*
* Summary:
*  Writes the specified number of bytes using the communications component (SPI)
*
* Parameters:  
*  wrData - Pointer to write data buffer
*  byteCnt - No. of bytes to be written 
*
* Return: 
*  Returns a flag to indicate whether the operation was successful or not
*
*
*******************************************************************************/
int WriteData(uint8* wrData, int byteCnt)
{
	uint32 timeOut =1;
	
	/* Clears TX and RX FIFOs and the status registers */
	SPIM_ClearFIFO();
	
	SPIM_PutArray(wrData, byteCnt);

 	/* Wait till read operation is complete or timeout  */
	while(!(SPIM_ReadTxStatus() & SPIM_STS_SPI_DONE))
	{
		timeOut++;
		/* Check for timeout and if so exit with communication error code*/
		if(timeOut == 0)
		{
			return(CYRET_ERR_COMM_MASK);
		}	
	}
	/* Give some delay before the next operation so that the slave can process the written data */
	CyDelay(30);

	return(CYRET_SUCCESS);
}


/*******************************************************************************
* Function Name: ReadData
********************************************************************************
*
* Summary:
*  Reads the specified number of bytes usign the communications component (SPI)
*
* Parameters:  
*  rdData - Pointer to read data buffer
*  byteCnt - No. of bytes to be read 
*
* Return: 
*  Returns a flag to indicate whether the operation was successful or not
*
*
*******************************************************************************/
int ReadData(uint8* rdData, int byteCnt)
{
	uint16 timeOut =1, i;
    
	SPIM_ClearTxBuffer();
	SPIM_ClearRxBuffer();
    SPIM_ClearFIFO();
    
	for(i=0; i<byteCnt; i++)
	{
		/* In order to read the response from the SPI Slave, the SPI Master has to initiate the 
		transfer. So, write as many '0' bytes as the expected no. of response bytes*/
		SPIM_WriteTxData(0u);
		
		/* Wait till read operation is complete or timeout  */
        while(!(SPIM_ReadTxStatus() & SPIM_STS_SPI_DONE))
		{
			timeOut++	;

			/* Check for timeout and if so exit with communication error code*/
			if(timeOut == 0)
			{
				return(CYRET_ERR_COMM_MASK);
			}
		}
		SPIM_ClearTxBuffer();
	}
    
	/* Read the data bytes */		
	while (byteCnt>0)
	{
		(*rdData)=SPIM_ReadRxData();
		byteCnt--;
		CyDelay(5);
		rdData++;
	}

	return(CYRET_SUCCESS);
}

/* [] END OF FILE */

