/*  --------------------------------------------------------------------------
* Copyright 2013, Cypress Semiconductor Corporation.
*
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international
* treaty provisions. Cypress hereby grants to licensee a personal,
* non-exclusive, non-transferable license to copy, use, modify, create
* derivative works of, and compile the Cypress Source Code and derivative
* works for the sole purpose of creating custom software in support of
* licensee product to be used only in conjunction with a Cypress integrated
* circuit as specified in the applicable agreement. Any reproduction,
* modification, translation, compilation, or representation of this
* software except as specified above is prohibited without the express
* written permission of Cypress.
* 
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED,
* WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising
* out of the application or use of any product or circuit described herein.
* Cypress does not authorize its products for use as critical components in
* life-support systems where a malfunction or failure may reasonably be
* expected to result in significant injury to the user. The inclusion of
* Cypress' product in a life-support systems application implies that the
* manufacturer assumes all risk of such use and in doing so indemnifies
* Cypress against all charges.
* 
* Use may be limited by and subject to the applicable Cypress software
* license agreement.
* -----------------------------------------------------------------------------
* Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
*
*****************************************************************************
*  Project Name: SPI_Bootloader_Host
*  Project Revision: 1.00
*  Software Version: PSoC Creator 2.2
*  Device Tested: CY8C5868AXI-LP035
*  Compilers Tested: ARM GCC
*  Related Hardware: CY8CKIT-050
*****************************************************************************
***************************************************************************** */

/*
* Project Description:
* This is a sample bootloader host program demonstrating PSoC 5LP bootloading PSoC 3.
* The project is tested using two DVKs - One with PSoC 3 on CY8CKIT-030 and the second one
* with PSoC 5LP on CY8CKIT-050. PSoC 3 must be programmed with the SPI_Bootloader Program attached
* with the app note.
*
* Connections Required
* PSoC 5LP DVK (CY8CKIT-050):
*  P5[1] - MOSI connected to P0[7] (MOSI) of CY8CKIT-030
*  P5[3] - MISO connected to P0[5] (MISO) of CY8CKIT-030
*  P5[5] - SCLK connected to P0[3] (SCLK) of CY8CKIT-030
*  P2[6:0] - connected to the Char LCD 
*
* PSoC 3 DVK : PSoC  3 intially programmed with SPI_Bootloader program.
*  P0[7] - MOSI connected to P5[1] (MOSI) of CY8CKIT-050
*  P0[5] - MISO connected to P5[3] (MISO) of CY8CKIT-050
*  P0[3] - SCLK connected to P5[5] (SCLK) of CY8CKIT-050
*  P2[6:0] - connected to the Char LCD
*
* Note that the GNDs of both DVKs should be connected together.
*
*
* A bootload function is defined in main.c: BootloadStringImage which uses Bootloader Host APIs
* to bootload the contents of .cyacd file.
*
* BootloadStringImage function requires each line of .cyacd to be stored as a seperate string in a string array.
* The total number of lines in the file should also be manually calculated and stored as #define LINE_CNT
* The string image of .cyacd file is stored in StringImage.h file.
*
* There are two bootloadable projects: Bootloadable_1 (displays “Hello”) and Bootloadable_2 (displays “Bye”) on a 
* character LCD. With successive bootload operation, alternate bootloadables will be programmed onto the target. 
*
* The following events happen on each trigger of the switch P6 [1] of CY8CKIT – 050 (PSoC 5LP):
*
* On first switch press, Bootloadable_1.cyacd file will be bootloaded onto the target PSoC 3. 
* On successful bootloading the message “Bootloaded - Hello” will be displayed on CY8CKIT – 050 LCD 
* and the message “Hello” will be displayed on CY8CKIT – 030 LCD.
*
* For subsequent bootloading operation press the switch P6 [1] on CY8CKIT – 030. This makes PSoC 3 to enter bootloader 
* and be ready to bootload a new application.
*
* On next switch press on CY8CKIT – 050, Bootloadable_2.cyacd file will be bootloaded onto the target PSoC 3. 
* On successful bootloading the message “Bootloaded - Bye” will be displayed on CY8CKIT – 050 LCD and the message “Bye” 
* will be displayed on CY8CKIT – 030 LCD.
***************************************************************************** */

#include <device.h>
#include "string.h"
#include <cybtldr_parse.h>
#include <cybtldr_command.h>
#include <communication_api.h>
#include <cybtldr_api.h>
#include "StringImage.h"

/* Macros to choose between PSoC 3 and PSoC 5LP as a target device */
#define PSoC_3       	0
#define PSoC_5LP     	1

/* If your target device is PSoC 5LP, change the value of this Macro to "PSoC_5LP" */
#define TARGET_DEVICE	PSoC_5LP

/* This function bootloads the .cyacd file. It sends command packets and flash data to the target.
   Based on the response from the target, it decides whether to continue bootloading. */
uint8 BootloadStringImage(const char *bootloadImagePtr[],unsigned int lineCount);

/* This structure contains function pointers to the four communication layer functions 
   contained in the communication_api.c / .h */
CyBtldr_CommunicationsData comm1 ;

/* switch_flag is set upon the user input (button on P6 [1]) */
uint8 switch_flag         = 0;

/* toggle alternates between the two bootloadable files (Hello and Bye) */
uint8 toggle              = 0;

int main()
{
	/* error holds the success / failure message of the bootload operation */
	uint8 error           = 0;

	/* Display the instruction to the user on how to start bootloading */
	LCD_Char_Start();	
	LCD_Char_Position(1,0);
	LCD_Char_PrintString("P6.1 to BL");
	
	/* Enable all interrupts */
	CYGlobalIntEnable;
	
	/* Initialize the communication structure element - comm1 */
	comm1.OpenConnection  = &OpenConnection;
	comm1.CloseConnection = &CloseConnection;
	comm1.ReadData        = &ReadData;
	comm1.WriteData       = &WriteData;
	comm1.MaxTransferSize = 64;
    
    /* Upon user input on P6 [1], the switch_flag is set in this ISR */
	ISR_Switch_Start();

    for(;;)
    {
	    /* If Switch is pressed start the bootloading operation */
		if(switch_flag ==1)
		{
			LCD_Char_ClearDisplay();
			LCD_Char_PrintString("Bootloading...  ");
            
			/* Select the Bootloadable files based on the target device */
			if(TARGET_DEVICE == PSoC_3)
			{
				/* Alternate between the two bootloadable files (Hello and Bye) based on toggle */	
	            if (toggle == 0)
				{   
					/* Bootload the Bootloadable_1.cyacd (Hello) */
	                error = BootloadStringImage(stringImage1,LINE_CNT_1);
	            }
				else if(toggle == 1)
	            {
					/* Bootload the Bootloadable_2.cyacd (Bye) */
				    error = BootloadStringImage(stringImage2,LINE_CNT_2);
	            }
            }
			else if(TARGET_DEVICE == PSoC_5LP)
			{
				/* Alternate between the two bootloadable files (Hello and Bye) based on toggle */	
	            if (toggle == 0)
				{   
					/* Bootload the Bootloadable_1.cyacd (Hello) */
	                error = BootloadStringImage(stringImage3,LINE_CNT_3);
	            }
				else if(toggle == 1)
	            {
					/* Bootload the Bootloadable_2.cyacd (Bye) */
				    error = BootloadStringImage(stringImage4,LINE_CNT_4);
	            }
			}
			else
			{
				/* Do nothing if the target device is neither PSoC 3 nor PSoC 5LP */
			}
			switch_flag=0;
			
			/* Check if the bootload operation is successful */
			if(error == CYRET_SUCCESS)
			{
				/* Display the success message */
				LCD_Char_Position(0,0);
                if(toggle == 0)
                {
				    LCD_Char_PrintString("Bootloaded-Hello");
                    LCD_Char_Position(1,0);
				    LCD_Char_PrintString("P6.1 to BL Bye  ");
	            	toggle = 1;
                }
                else
                {
				    LCD_Char_PrintString("Bootloaded-Bye");
                    LCD_Char_Position(1,0);
			        LCD_Char_PrintString("P6.1 to BL Hello  ");
					toggle = 0;
                }
			}
			else 
			{
				/* Display the failure message along with the approiate error code */
				if(error & CYRET_ERR_COMM_MASK) /* Check for comm error*/
				{
					LCD_Char_Position(0,0);
					LCD_Char_PrintString("Communicatn Err ");
				}
				else /* Else Display the bootload error code */
				{
					LCD_Char_Position(0,0);
					LCD_Char_PrintString("Bootload Err :");
					LCD_Char_PrintHexUint8(error);
				}
			}	
		}
		else 
		{
			/* Do Nothing */
		}
	}	
}

/*******************************************************************************
* Function Name: BootloadStringImage
********************************************************************************
*
* Summary:
*  Bootloads the .cyacd file contents which is stored as string array
*
* Parameters:  
* bootloadImagePtr - Pointer to the string array
* lineCount - No. of lines in the .cyacd file(No: of rows in the string array)
*
* Return: 
*  Returns a flag to indicate whether the bootload operation was successful or not
*
*
*******************************************************************************/
uint8 BootloadStringImage(const char *bootloadImagePtr[],unsigned int lineCount )
{
	unsigned char err;
	unsigned char arrayId; 
	unsigned short rowNum;
	unsigned short rowSize; 
	unsigned char checksum ;
	unsigned char checksum2;
	unsigned long blVer=0;
	
	/* rowData buffer size should be equal to the length of data to be sent to each flash row
	*  Each flash row has 288 bytes if ECC is disabled, or 256 bytes + 32 ECC (if enabled) bytes */
	unsigned char rowData[288];
	unsigned int lineLen;
	unsigned long  siliconID;
	unsigned char siliconRev;
	unsigned char packetChkSumType;
	unsigned int lineCntr ;
	
	/* Initialize line counter */
	lineCntr = 0;
	
	/* Get length of the first line in cyacd file*/
	lineLen = strlen(bootloadImagePtr[lineCntr]);

	/* Parse the first line (header) of *.cyacd file to extract siliconID, siliconRev and packetChkSumType */
	err = CyBtldr_ParseHeader(lineLen ,(unsigned char *)bootloadImagePtr[lineCntr] , &siliconID , &siliconRev ,&packetChkSumType);
    
	/* Set the packet checksum type for communicating with bootloader. The packet checksum type to be used 
	* is determined from the *.cyacd file header information */
	CyBtldr_SetCheckSumType((CyBtldr_ChecksumType)packetChkSumType);
	
	if(err==CYRET_SUCCESS)
	{
		/* Start Bootload operation */
		err = CyBtldr_StartBootloadOperation(&comm1 ,siliconID, siliconRev ,&blVer);
		lineCntr++ ;
		while((err == CYRET_SUCCESS)&& (lineCntr <  lineCount))
		{
            /* Get the string length for the line*/
			lineLen =  strlen(bootloadImagePtr[lineCntr]);
			/*Parse row data*/
			err = CyBtldr_ParseRowData((unsigned int)lineLen,(unsigned char *)bootloadImagePtr[lineCntr], &arrayId, &rowNum, rowData, &rowSize, &checksum);
          
			if (CYRET_SUCCESS == err)
            {
				/* Program Row */
				err = CyBtldr_ProgramRow(arrayId, rowNum, rowData, rowSize);
	            if (CYRET_SUCCESS == err)
				{
					/* Verify Row . Check whether the checksum received from bootloader matches
					* the expected row checksum stored in cyacd file*/
					checksum2 = (unsigned char)(checksum + arrayId + rowNum + (rowNum >> 8) + rowSize + (rowSize >> 8));
					err = CyBtldr_VerifyRow(arrayId, rowNum, checksum2);
				}
            }
			
			/* Increment the linCntr */
			lineCntr++;
		}
		
		/* End Bootloader Operation */
		CyBtldr_EndBootloadOperation();
	}
	return(err);
}


/* [] END OF FILE */
